import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeElevenReport:
    """Class provides the definitions for Fault Condition 11 Report."""

    def __init__(self, config):
        self.sat_setpoint_col = config["SAT_SETPOINT_COL"]
        self.oat_col = config["OAT_COL"]
        self.cooling_sig_col = config["COOLING_SIG_COL"]
        self.economizer_sig_col = config["ECONOMIZER_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc11_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 11 Plot")

        ax1.plot(df.index, df[self.sat_setpoint_col], label="SATSP")
        ax1.plot(df.index, df[self.oat_col], label="OAT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[self.cooling_sig_col], label="AHU Cool Vlv", color="r")
        ax2.plot(df.index, df[self.economizer_sig_col], label="AHU Dpr Cmd", color="g")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc11_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc11_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_oat": round(
                df[self.oat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat_sp": round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc11_flag"

        df["hour_of_the_day_fc11"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc11.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 11 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(
        self, df: pd.DataFrame, output_col: str = "fc11_flag"
    ):
        print(
            "Fault Condition 11: Outside air temperature too low for 100% outside air cooling in economizer mode"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_oat = round(df[self.oat_col].where(df[output_col] == 1).mean(), 2)
            print("Outside Air Temp Mean When In Fault: ", flag_true_oat)

            flag_true_sat_sp = round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            )
            print("Supply Air Temp Setpoint Mean When In Fault: ", flag_true_sat_sp)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high, indicating temperature sensor error or the heating coil could be leaking, potentially creating simultaneous heating/cooling scenarios. Visually verify damper operation, and consider tuning the BAS programming for proper AHU operation."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the AHU components are within calibration for this fault equation."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_six import FaultConditionSix
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc6.py -rP -s

OA FRACTION TOO LOW OR TOO HIGH; SHOULD BE EQUAL TO %OAmin
"""

# Constants
TEST_AIRFLOW_ERR_THRES = 0.3
TEST_AHU_MIN_CFM_DESIGN = 3000
TEST_OAT_DEGF_ERR_THRES = 5.0
TEST_RAT_DEGF_ERR_THRES = 2.0
TEST_DELTA_TEMP_MIN = 10.0
TEST_AHU_MIN_OA_DPR = 0.2
TEST_VAV_TOTAL_AIR_FLOW_COL = "vav_total_air_flow"
TEST_MIX_TEMP_COL = "mix_air_temp"
TEST_OUT_TEMP_COL = "out_air_temp"
TEST_RETURN_TEMP_COL = "return_air_temp"
TEST_SUPPLY_VFD_SPEED_COL = "fan_vfd_speed_col"
TEST_MIX_AIR_DAMPER_COL = "economizer_sig_col"
TEST_HEATING_COIL_SIG_COL = "heating_sig_col"
TEST_COOLING_COIL_SIG_COL = "cooling_sig_col"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionSix with a dictionary
fault_condition_params = {
    "AIRFLOW_ERR_THRES": TEST_AIRFLOW_ERR_THRES,
    "AHU_MIN_OA_CFM_DESIGN": TEST_AHU_MIN_CFM_DESIGN,
    "OUTDOOR_DEGF_ERR_THRES": TEST_OAT_DEGF_ERR_THRES,
    "RETURN_DEGF_ERR_THRES": TEST_RAT_DEGF_ERR_THRES,
    "OAT_RAT_DELTA_MIN": TEST_DELTA_TEMP_MIN,
    "AHU_MIN_OA_DPR": TEST_AHU_MIN_OA_DPR,
    "SUPPLY_FAN_AIR_VOLUME_COL": TEST_VAV_TOTAL_AIR_FLOW_COL,
    "MAT_COL": TEST_MIX_TEMP_COL,
    "OAT_COL": TEST_OUT_TEMP_COL,
    "RAT_COL": TEST_RETURN_TEMP_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "ECONOMIZER_SIG_COL": TEST_MIX_AIR_DAMPER_COL,
    "HEATING_SIG_COL": TEST_HEATING_COIL_SIG_COL,
    "COOLING_SIG_COL": TEST_COOLING_COIL_SIG_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,  # rolling sum window size
}

fc6 = FaultConditionSix(fault_condition_params)


class TestNoFaultNoHtg(object):

    def no_fault_df_no_htg(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [55, 56, 55.5, 56.5, 55.2, 55.8],
            TEST_OUT_TEMP_COL: [10, 10.5, 10.2, 10.8, 10.3, 10.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        }
        return pd.DataFrame(data)

    def test_no_fault_no_htg(self):
        results = fc6.apply(self.no_fault_df_no_htg())
        actual = results["fc6_flag"].sum()
        expected = 0
        message = (
            f"FC6 no_fault_df_no_htg actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestFaultInHtg(object):

    def fault_df_in_htg(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [30, 29.5, 30.5, 29.8, 30.2, 29.6],
            TEST_OUT_TEMP_COL: [10, 10.5, 10.2, 10.8, 10.3, 10.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_COOLING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
        }
        return pd.DataFrame(data)

    def test_fault_in_htg(self):
        results = fc6.apply(self.fault_df_in_htg())
        actual = results["fc6_flag"].sum()
        expected = 2
        message = f"FC6 fault_df_in_htg actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestNoFaultInEconMode(object):

    def no_fault_df_in_econ(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [90, 91, 89.5, 91.2, 90.5, 91.1],
            TEST_OUT_TEMP_COL: [110, 110.5, 110.2, 110.8, 110.3, 110.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [0.99, 0.98, 0.97, 0.99, 0.98, 0.99],
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.50, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_no_fault_in_econ_mode(self):
        results = fc6.apply(self.no_fault_df_in_econ())
        actual = results["fc6_flag"].sum()
        expected = 0
        message = (
            f"FC6 no_fault_df_in_econ actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestNoFaultInEconPlusMechClg(object):

    def no_fault_df_in_econ_plus_mech(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [55, 56, 55.5, 56.5, 55.2, 55.8],
            TEST_OUT_TEMP_COL: [10, 10.5, 10.2, 10.8, 10.3, 10.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.5, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_no_fault_in_econ_plus_mech_mode(self):
        results = fc6.apply(self.no_fault_df_in_econ_plus_mech())
        actual = results["fc6_flag"].sum()
        expected = 0
        message = f"FC6 no_fault_df_in_econ_plus_mech actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultInMechClg(object):

    def fault_df_in_mech_clg(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [30, 30.5, 30, 30.5, 30.8, 30.2],
            TEST_OUT_TEMP_COL: [5.2, 5.5, 5.2, 5.8, 5.3, 5.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.50, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_fault_in_mech_mode(self):
        results = fc6.apply(self.fault_df_in_mech_clg())
        actual = results["fc6_flag"].sum()
        expected = 2
        message = (
            f"FC6 fault_df_in_mech_clg actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestNoFaultInMechClg(object):

    def no_fault_df_in_mech_clg(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [60, 60.5, 60, 60.5, 60.8, 60.2],
            TEST_OUT_TEMP_COL: [5.2, 5.5, 5.2, 5.8, 5.3, 5.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [0.66, 0.67, 0.65, 0.66, 0.68, 0.67],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.50, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_no_fault_in_mech_mode(self):
        results = fc6.apply(self.no_fault_df_in_mech_clg())
        actual = results["fc6_flag"].sum()
        expected = 0
        message = (
            f"FC6 no_fault_df_in_mech_clg actual is {actual} and expected is {expected}"
        )
        assert actual == expected, message


class TestFaultOnInt(object):

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [80, 80.5, 81, 80.8, 81.2, 80.3],
            TEST_OUT_TEMP_COL: [110, 110.5, 110.2, 110.8, 110.3, 110.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [66, 67, 65, 66, 68, 67],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.50, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc6.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne(object):

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_VAV_TOTAL_AIR_FLOW_COL: [10000, 10050, 10025, 10075, 10030, 10020],
            TEST_MIX_TEMP_COL: [80, 80.5, 81, 80.8, 81.2, 80.3],
            TEST_OUT_TEMP_COL: [110, 110.5, 110.2, 110.8, 110.3, 110.1],
            TEST_RETURN_TEMP_COL: [72, 72.5, 72.2, 72.8, 72.3, 72.1],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 1.2, 1.3, 1.1, 1.2, 1.1],
            TEST_MIX_AIR_DAMPER_COL: [TEST_AHU_MIN_OA_DPR] * 6,
            TEST_HEATING_COIL_SIG_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_COOLING_COIL_SIG_COL: [0.50, 0.51, 0.49, 0.52, 0.50, 0.51],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc6.apply(self.fault_df_on_output_greater_than_one())


if __name__ == "__main__":
    pytest.main()



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeTenReport:
    """Class provides the definitions for Fault Condition 10 Report."""

    def __init__(self, config):
        self.oat_col = config["OAT_COL"]
        self.mat_col = config["MAT_COL"]
        self.cooling_sig_col = config["COOLING_SIG_COL"]
        self.economizer_sig_col = config["ECONOMIZER_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc10_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 10 Plot")

        ax1.plot(df.index, df[self.mat_col], label="MAT")
        ax1.plot(df.index, df[self.oat_col], label="OAT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[self.cooling_sig_col], label="AHU Cool Vlv", color="r")
        ax2.plot(df.index, df[self.economizer_sig_col], label="AHU Dpr Cmd", color="g")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc10_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc10_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_oat": round(
                df[self.oat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc10_flag"

        df["hour_of_the_day_fc10"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc10.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 10 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(
        self, df: pd.DataFrame, output_col: str = "fc10_flag"
    ):
        print(
            "Fault Condition 10: Outdoor air temperature and mix air temperature should be approximately equal in economizer plus mech cooling mode"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_oat = round(df[self.oat_col].where(df[output_col] == 1).mean(), 2)
            print("Outside Air Temp Mean When In Fault: ", flag_true_oat)

            flag_true_mat = round(df[self.mat_col].where(df[output_col] == 1).mean(), 2)
            print("Mix Air Temp Mean When In Fault: ", flag_true_mat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high indicating temperature sensor error or the mixing air dampers are stuck or broken with the inability for the AHU to go into a proper 100 percent outside air mode. Verify the actual installation of temperature sensors, damper operation, and related equipment to address potential issues."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low indicating the AHU components are within calibration for this fault equation."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeThreeReport:
    def __init__(self, config):
        self.mix_degf_err_thres = config["MIX_DEGF_ERR_THRES"]
        self.return_degf_err_thres = config["RETURN_DEGF_ERR_THRES"]
        self.outdoor_degf_err_thres = config["OUTDOOR_DEGF_ERR_THRES"]
        self.mat_col = config["MAT_COL"]
        self.rat_col = config["RAT_COL"]
        self.oat_col = config["OAT_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc3_flag"

        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 3 Plot")

        ax1.plot(df.index, df[self.mat_col], color="r", label="Mix Temp")
        ax1.plot(df.index, df[self.rat_col], color="b", label="Return Temp")
        ax1.plot(df.index, df[self.oat_col], color="g", label="Out Temp")
        ax1.legend(loc="best")
        ax1.set_ylabel("°F")

        ax2.plot(df.index, df[output_col], label="Fault", color="k")
        ax2.set_xlabel("Date")
        ax2.set_ylabel("Fault Flags")
        ax2.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc3_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc3_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_oat": round(
                df[self.oat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_rat": round(
                df[self.rat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc3_flag"

        df["hour_of_the_day_fc3"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc3.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 3 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc3_flag"):
        print(
            "Fault Condition 3: Mix temperature too high; should be between outside and return air"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            print("Mix Air Temp Mean When In Fault: ", summary["flag_true_mat"])
            print("Outside Air Temp Mean When In Fault: ", summary["flag_true_oat"])
            print("Return Temp Mean When In Fault: ", summary["flag_true_rat"])

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high, indicating potential issues with the mix temperature. Verify sensor calibration and investigate possible mechanical problems."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the system is likely functioning correctly."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_four import FaultConditionFour
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
from datetime import datetime, timezone

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest open_fdd/tests/ahu/test_ahu_fc4.py -rP -s

Too much hunting in control system
OS state changes greater than 7 in an hour
"""

# Constants
DELTA_OS_MAX = 7
AHU_MIN_OA = 0.20
TEST_MIX_AIR_DAMPER_COL = "economizer_sig_col"
TEST_HEATING_COIL_SIG_COL = "heating_sig_col"
TEST_COOLING_COIL_SIG_COL = "cooling_sig_col"
TEST_SUPPLY_VFD_SPEED_COL = "fan_vfd_speed_col"
TEST_DATASET_ROWS = 60

# Initialize FaultConditionFour with a dictionary
fault_condition_params = {
    "DELTA_OS_MAX": DELTA_OS_MAX,
    "AHU_MIN_OA_DPR": AHU_MIN_OA,
    "ECONOMIZER_SIG_COL": TEST_MIX_AIR_DAMPER_COL,
    "HEATING_SIG_COL": TEST_HEATING_COIL_SIG_COL,
    "COOLING_SIG_COL": TEST_COOLING_COIL_SIG_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,  # default value
}

fc4 = FaultConditionFour(fault_condition_params)


def generate_timestamp() -> pd.Series:
    df = pd.DataFrame()
    date_range = pd.period_range(
        # make a time stamp starting at top of
        # the hour with one min intervals
        start=datetime(2022, 6, 6, 14, 30, 0, 0, tzinfo=timezone.utc),
        periods=TEST_DATASET_ROWS,
        freq="min",
    )
    df["Date"] = [x.to_timestamp() for x in date_range]
    return df["Date"]


def econ_plus_mech_clg_row() -> dict:
    data = {
        TEST_MIX_AIR_DAMPER_COL: 0.6,
        TEST_HEATING_COIL_SIG_COL: 0.0,
        TEST_COOLING_COIL_SIG_COL: 0.6,
        TEST_SUPPLY_VFD_SPEED_COL: 0.8,
    }
    return data


def mech_clg_row() -> dict:
    data = {
        TEST_MIX_AIR_DAMPER_COL: 0.0,
        TEST_HEATING_COIL_SIG_COL: 0.0,
        TEST_COOLING_COIL_SIG_COL: 0.6,
        TEST_SUPPLY_VFD_SPEED_COL: 0.8,
    }
    return data


def econ_plus_mech_clg_row_int() -> dict:
    data = {
        TEST_MIX_AIR_DAMPER_COL: 0.6,
        TEST_HEATING_COIL_SIG_COL: 0.0,
        TEST_COOLING_COIL_SIG_COL: 0.6,
        TEST_SUPPLY_VFD_SPEED_COL: 88,
    }
    return data


def econ_plus_mech_clg_row_float_greater_than_one() -> dict:
    data = {
        TEST_MIX_AIR_DAMPER_COL: 0.6,
        TEST_HEATING_COIL_SIG_COL: 0.0,
        TEST_COOLING_COIL_SIG_COL: 0.6,
        TEST_SUPPLY_VFD_SPEED_COL: 88.8,
    }
    return data


class TestFault(object):

    def fault_df(self) -> pd.DataFrame:
        data = []
        counter = 0
        for i in range(TEST_DATASET_ROWS):
            if i % 2 == 0 and counter < 11:
                data.append(econ_plus_mech_clg_row())
                counter += 1  # only simulate 10 OS changes
            else:
                data.append(mech_clg_row())
        return pd.DataFrame(data)

    def test_fault(self):
        fault_df = self.fault_df().set_index(generate_timestamp())
        results = fc4.apply(fault_df)
        actual = results["fc4_flag"].sum()
        expected = 1
        message = f"FC4 fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestNoFault(object):

    def no_fault_df(self) -> pd.DataFrame:
        data = []
        for i in range(TEST_DATASET_ROWS):
            data.append(mech_clg_row())
        return pd.DataFrame(data)

    def test_no_fault(self):
        no_fault_df = self.no_fault_df().set_index(generate_timestamp())
        results = fc4.apply(no_fault_df)
        actual = results["fc4_flag"].sum()
        expected = 0
        message = f"FC4 no_fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt(object):

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = []
        for i in range(TEST_DATASET_ROWS):
            if i % 2 == 0:
                data.append(econ_plus_mech_clg_row_int())
            else:
                data.append(mech_clg_row())
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        fault_df_on_output_int = self.fault_df_on_output_int().set_index(
            generate_timestamp()
        )
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc4.apply(fault_df_on_output_int)


class TestFaultOnFloatGreaterThanOne(object):

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = []
        for i in range(TEST_DATASET_ROWS):
            if i % 2 == 0:
                data.append(econ_plus_mech_clg_row_float_greater_than_one())
            else:
                data.append(mech_clg_row())
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        fault_df_on_output_greater_than_one = (
            self.fault_df_on_output_greater_than_one().set_index(generate_timestamp())
        )
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc4.apply(fault_df_on_output_greater_than_one)


class TestFaultOnMixedTypes(object):

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = []
        for i in range(TEST_DATASET_ROWS):
            if i % 2 == 0:
                data.append(
                    {
                        TEST_MIX_AIR_DAMPER_COL: 0.6,
                        TEST_HEATING_COIL_SIG_COL: 0.0,
                        TEST_COOLING_COIL_SIG_COL: 0.6,
                        TEST_SUPPLY_VFD_SPEED_COL: 1.1,
                    }
                )
            else:
                data.append(mech_clg_row())
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        fault_df_on_mixed_types = self.fault_df_on_mixed_types().set_index(
            generate_timestamp()
        )
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc4.apply(fault_df_on_mixed_types)



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeNineReport:
    """Class provides the definitions for Fault Condition 9 Report."""

    def __init__(self, config):
        self.sat_setpoint_col = config["SAT_SETPOINT_COL"]
        self.oat_col = config["OAT_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]
        self.economizer_sig_col = config["ECONOMIZER_SIG_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc9_flag"

        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 9 Plot")

        ax1.plot(df.index, df[self.sat_setpoint_col], label="SATSP")
        ax1.plot(df.index, df[self.oat_col], label="OAT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[output_col], label="Fault", color="k")
        ax2.set_xlabel("Date")
        ax2.set_ylabel("Fault Flags")
        ax2.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc9_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc9_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_oat": round(
                df[self.oat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_satsp": round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc9_flag"

        df["hour_of_the_day_fc9"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc9.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 9 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc9_flag"):
        print(
            "Fault Condition 9: Outside air temperature too high in free cooling without additional mechanical cooling in economizer mode"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_oat = round(df[self.oat_col].where(df[output_col] == 1).mean(), 2)
            print("Outside Air Temp Mean When In Fault: ", flag_true_oat)

            flag_true_satsp = round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            )
            print("Supply Air Temp Setpoint Mean When In Fault: ", flag_true_satsp)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high indicating temperature sensor error or the cooling valve is stuck open or leaking causing overcooling. Trouble shoot a leaking valve by isolating the coil with manual shutoff valves and verify a change in AHU discharge air temperature with the AHU running."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low indicating the AHU components are within calibration for this fault equation."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeOneReport:
    def __init__(self, config):
        self.vfd_speed_percent_err_thres = config["VFD_SPEED_PERCENT_ERR_THRES"]
        self.duct_static_col = config["DUCT_STATIC_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]
        self.duct_static_setpoint_col = config["DUCT_STATIC_SETPOINT_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc1_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 1 Plot")

        ax1.plot(df.index, df[self.duct_static_col], label="STATIC")
        ax1.legend(loc="best")
        ax1.set_ylabel("Inch WC")

        ax2.plot(df.index, df[self.supply_vfd_speed_col], color="g", label="FAN")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc1_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc1_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_duct_static": round(
                df[self.duct_static_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_duct_static_spt": round(
                df[self.duct_static_setpoint_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc1_flag"

        df["hour_of_the_day_fc1"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc1.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 1 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc1_flag"):
        print("Fault Condition 1: Duct static too low at fan at full speed")

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            print("Duct Static Mean When In Fault: ", summary["flag_true_duct_static"])
            print(
                "Duct Static Setpoint Mean When In Fault: ",
                summary["flag_true_duct_static_spt"],
            )

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high, indicating potential duct issues. Verify system calibration and investigate possible mechanical problems."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the system is likely functioning correctly."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_two import FaultConditionTwo
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc2.py -rP -s

Mix air temp lower than out temp
"""

TEST_OUTDOOR_DEGF_ERR_THRES = 5.0
TEST_MIX_DEGF_ERR_THRES = 2.0
TEST_RETURN_DEGF_ERR_THRES = 2.0
TEST_MIX_TEMP_COL = "mix_air_temp"
TEST_RETURN_TEMP_COL = "return_air_temp"
TEST_OUT_TEMP_COL = "out_air_temp"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
TEST_ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionTwo with a dictionary
fault_condition_params = {
    "MIX_DEGF_ERR_THRES": TEST_MIX_DEGF_ERR_THRES,
    "RETURN_DEGF_ERR_THRES": TEST_RETURN_DEGF_ERR_THRES,
    "OUTDOOR_DEGF_ERR_THRES": TEST_OUTDOOR_DEGF_ERR_THRES,
    "MAT_COL": TEST_MIX_TEMP_COL,
    "RAT_COL": TEST_RETURN_TEMP_COL,
    "OAT_COL": TEST_OUT_TEMP_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": TEST_ROLLING_WINDOW_SIZE,
}

fc2 = FaultConditionTwo(fault_condition_params)


class TestNoFault(object):

    def no_fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [60.0, 62.0, 64.0, 61.0, 63.0, 60.0],
            TEST_RETURN_TEMP_COL: [72.0, 73.0, 74.0, 72.0, 73.0, 72.0],
            TEST_OUT_TEMP_COL: [45.0, 46.0, 47.0, 45.0, 46.0, 45.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.8, 0.82, 0.85, 0.8, 0.83, 0.8],
        }
        return pd.DataFrame(data)

    def test_no_fault(self):
        results = fc2.apply(self.no_fault_df())
        actual = results["fc2_flag"].sum()
        expected = 0
        message = f"fc2 no_fault_df actual is {actual} and expected is {expected}"
        print(message)
        assert actual == expected, message


class TestFault(object):

    def fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [45.0, 46.0, 45.0, 46.0, 45.0, 45.0],
            TEST_RETURN_TEMP_COL: [72.0, 72.5, 73.0, 72.0, 72.5, 72.0],
            TEST_OUT_TEMP_COL: [60.0, 60.5, 61.0, 60.0, 60.5, 60.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.8, 0.82, 0.81, 0.8, 0.82, 0.8],
        }
        return pd.DataFrame(data)

    def test_fault(self):
        results = fc2.apply(self.fault_df())
        actual = results["fc2_flag"].sum()
        expected = 2
        message = f"fc2 fault_df actual is {actual} and expected is {expected}"
        print(message)
        assert actual == expected, message


class TestFaultOnInt(object):

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [45.0, 46.0, 45.0, 46.0, 45.0, 45.0],
            TEST_RETURN_TEMP_COL: [72.0, 72.5, 73.0, 72.0, 72.5, 72.0],
            TEST_OUT_TEMP_COL: [60.0, 60.5, 61.0, 60.0, 60.5, 60.0],
            TEST_SUPPLY_VFD_SPEED_COL: [88, 88, 88, 88, 88, 88],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc2.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne(object):

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [45.0, 46.0, 45.0, 46.0, 45.0, 45.0],
            TEST_RETURN_TEMP_COL: [72.0, 72.5, 73.0, 72.0, 72.5, 72.0],
            TEST_OUT_TEMP_COL: [60.0, 60.5, 61.0, 60.0, 60.5, 60.0],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 1.2, 1.3, 1.1, 1.2, 1.1],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc2.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes(object):

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [45.0, 46.0, 45.0, 46.0, 45.0, 45.0],
            TEST_RETURN_TEMP_COL: [72.0, 72.5, 73.0, 72.0, 72.5, 72.0],
            TEST_OUT_TEMP_COL: [60.0, 60.5, 61.0, 60.0, 60.5, 60.0],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 0.82, 1.3, 1.1, 0.82, 1.1],
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc2.apply(self.fault_df_on_mixed_types())



import pandas as pd
import pandas.api.types as pdtypes
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionFive(FaultCondition):
    """Class provides the definitions for Fault Condition 5.
    SAT too low; should be higher than MAT in HTG MODE
    --Broken heating valve or other mechanical issue
    related to heat valve not working as designed
    """

    def __init__(self, dict_):
        self.mix_degf_err_thres = float
        self.supply_degf_err_thres = float
        self.delta_t_supply_fan = float
        self.mat_col = str
        self.sat_col = str
        self.heating_sig_col = str
        self.supply_vfd_speed_col = str
        self.troubleshoot_mode = bool  # default to False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    # fault only active if fan is running and htg vlv is modulating
    # OS 1 is heating mode only fault
    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # check analog outputs [data with units of %] are floats only
        columns_to_check = [self.supply_vfd_speed_col, self.heating_sig_col]

        for col in columns_to_check:
            self.check_analog_pct(df, [col])

        df["sat_check"] = df[self.sat_col] + self.supply_degf_err_thres
        df["mat_check"] = (
            df[self.mat_col] - self.mix_degf_err_thres + self.delta_t_supply_fan
        )

        df["combined_check"] = (
            (df["sat_check"] <= df["mat_check"])
            # this is to make fault only active in OS1 for htg mode only
            # and fan is running. Some control programming may use htg
            # vlv when AHU is off to prevent low limit freeze alarms
            & (df[self.heating_sig_col] > 0.01)
            & (df[self.supply_vfd_speed_col] > 0.01)
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc5_flag"] = (rolling_sum == self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["mat_check"]
            del df["sat_check"]
            del df["combined_check"]

        return df



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeFourReport:
    """Class provides the definitions for Fault Code 4 Report.
    Reporting the time series average dataframe that calculates control states per hour.
    """

    def __init__(self, config):
        self.delta_os_max = config["DELTA_OS_MAX"]
        self.heating_mode_calc_col = "heating_mode"
        self.econ_only_cooling_mode_calc_col = "econ_only_cooling_mode"
        self.econ_plus_mech_cooling_mode_calc_col = "econ_plus_mech_cooling_mode"
        self.mech_cooling_only_mode_calc_col = "mech_cooling_only_mode"

    def summarize_fault_times(
        self, df: pd.DataFrame, output_col: str = "fc4_flag"
    ) -> dict:
        delta_all_data = df.index.to_series().diff()

        total_days_all_data = round(delta_all_data.sum() / pd.Timedelta(days=1), 2)
        total_hours_all_data = round(delta_all_data.sum() / pd.Timedelta(hours=1))
        hours_fc4_mode = round(
            (delta_all_data * df[output_col]).sum() / pd.Timedelta(hours=1)
        )
        percent_true_fc4 = round(df[output_col].mean() * 100, 2)
        percent_false_fc4 = round((100 - percent_true_fc4), 2)

        # Heating mode runtime stats
        delta_heating = df[self.heating_mode_calc_col].index.to_series().diff()
        total_hours_heating = (
            delta_heating * df[self.heating_mode_calc_col]
        ).sum() / pd.Timedelta(hours=1)
        percent_heating = round(df[self.heating_mode_calc_col].mean() * 100, 2)

        # Econ mode runtime stats
        delta_econ = df[self.econ_only_cooling_mode_calc_col].index.to_series().diff()
        total_hours_econ = (
            delta_econ * df[self.econ_only_cooling_mode_calc_col]
        ).sum() / pd.Timedelta(hours=1)
        percent_econ = round(df[self.econ_only_cooling_mode_calc_col].mean() * 100, 2)

        # Econ plus mech cooling mode runtime stats
        delta_econ_clg = (
            df[self.econ_plus_mech_cooling_mode_calc_col].index.to_series().diff()
        )
        total_hours_econ_clg = (
            delta_econ_clg * df[self.econ_plus_mech_cooling_mode_calc_col]
        ).sum() / pd.Timedelta(hours=1)
        percent_econ_clg = round(
            df[self.econ_plus_mech_cooling_mode_calc_col].mean() * 100, 2
        )

        # Mech cooling mode runtime stats
        delta_clg = df[self.mech_cooling_only_mode_calc_col].index.to_series().diff()
        total_hours_clg = (
            delta_clg * df[self.mech_cooling_only_mode_calc_col]
        ).sum() / pd.Timedelta(hours=1)
        percent_clg = round(df[self.mech_cooling_only_mode_calc_col].mean() * 100, 2)

        return {
            "total_days": total_days_all_data,
            "total_hours": total_hours_all_data,
            "hours_in_fault": hours_fc4_mode,
            "percent_of_time_in_fault": percent_true_fc4,
            "percent_of_time_not_in_fault": percent_false_fc4,
            "percent_of_time_AHU_in_mech_clg_mode": percent_clg,
            "percent_of_time_AHU_in_econ_plus_mech_clg_mode": percent_econ_clg,
            "percent_of_time_AHU_in_econ_free_clg_mode": percent_econ,
            "percent_of_time_AHU_in_heating_mode": percent_heating,
            "total_hours_heating_mode": total_hours_heating,
            "total_hours_econ_mode": total_hours_econ,
            "total_hours_econ_mech_clg_mode": total_hours_econ_clg,
            "total_hours_mech_clg_mode": total_hours_clg,
        }

    def create_plot(self, df: pd.DataFrame, output_col: str = "fc4_flag"):
        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(25, 8))
        fig.suptitle("Fault Condition 4 Plots")

        ax1.plot(df.index, df[self.heating_mode_calc_col], label="Heat", color="orange")
        ax1.plot(
            df.index,
            df[self.econ_only_cooling_mode_calc_col],
            label="Econ Clg",
            color="olive",
        )
        ax1.plot(
            df.index,
            df[self.econ_plus_mech_cooling_mode_calc_col],
            label="Econ + Mech Clg",
            color="c",
        )
        ax1.plot(
            df.index,
            df[self.mech_cooling_only_mode_calc_col],
            label="Mech Clg",
            color="m",
        )
        ax1.set_xlabel("Date")
        ax1.set_ylabel("Calculated AHU Operating States")
        ax1.legend(loc="best")

        ax2.plot(df.index, df[output_col], label="Fault", color="k")
        ax2.set_xlabel("Date")
        ax2.set_ylabel("Fault Flags")
        ax2.legend(loc="best")

        plt.tight_layout()
        plt.show()
        plt.close()

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = "fc4_flag"):
        df["hour_of_the_day"] = df.index.hour.where(df[output_col] == 1)
        df = df.dropna(subset=["hour_of_the_day"])
        print("\nTime-of-day Histogram Data")
        print(df["hour_of_the_day"])
        print()
        sys.stdout.flush()

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day.dropna(), bins=24)
        ax.set_xlabel("Hour of the Day")
        ax.set_ylabel("Frequency")
        ax.set_title("Hour-Of-Day When Fault Flag is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc4_flag"):
        print("Fault Condition 4: Hunting too many OS state changes")

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)
        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)
            sys.stdout.flush()



import sys
from open_fdd.air_handling_unit.faults.shared_utils import SharedUtils
import pandas as pd


class HelperUtils:
    def float_int_check_err(self, col):
        return SharedUtils.float_int_check_err(col)

    def float_max_check_err(self, col):
        return SharedUtils.float_max_check_err(col)

    def isfloat(self, num):
        return SharedUtils.isfloat(num)

    def isLessThanOnePointOne(self, num):
        return SharedUtils.isLessThanOnePointOne(num)

    def convert_to_float(self, df, col):
        return SharedUtils.convert_to_float(df, col)

    def apply_rolling_average_if_needed(self, df, freq="1min", rolling_window="5min"):
        return SharedUtils.apply_rolling_average_if_needed(df, freq, rolling_window)

    def process_all_faults(self, df, config_dict):
        # Import fault conditions
        from open_fdd.air_handling_unit.faults.fault_condition_one import (
            FaultConditionOne,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_two import (
            FaultConditionTwo,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_three import (
            FaultConditionThree,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_four import (
            FaultConditionFour,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_five import (
            FaultConditionFive,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_six import (
            FaultConditionSix,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_seven import (
            FaultConditionSeven,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_eight import (
            FaultConditionEight,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_nine import (
            FaultConditionNine,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_ten import (
            FaultConditionTen,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_eleven import (
            FaultConditionEleven,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_twelve import (
            FaultConditionTwelve,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_thirteen import (
            FaultConditionThirteen,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_fourteen import (
            FaultConditionFourteen,
        )
        from open_fdd.air_handling_unit.faults.fault_condition_fifteen import (
            FaultConditionFifteen,
        )

        fault_counts = {}

        # Apply rolling average if needed
        df = self.apply_rolling_average_if_needed(df)

        # Initialize Fault Condition Classes
        fc1 = FaultConditionOne(config_dict)
        fc2 = FaultConditionTwo(config_dict)
        fc3 = FaultConditionThree(config_dict)
        fc4 = FaultConditionFour(config_dict)
        fc5 = FaultConditionFive(config_dict)
        fc7 = FaultConditionSeven(config_dict)
        fc8 = FaultConditionEight(config_dict)
        fc9 = FaultConditionNine(config_dict)
        fc10 = FaultConditionTen(config_dict)
        fc11 = FaultConditionEleven(config_dict)
        fc12 = FaultConditionTwelve(config_dict)
        fc13 = FaultConditionThirteen(config_dict)

        # Optionally initialize Fault Condition Six
        fc6 = None
        if config_dict.get("SUPPLY_FAN_AIR_VOLUME_COL") is not None:
            fc6 = FaultConditionSix(config_dict)

        # Optionally initialize Fault Condition Fourteen
        fc14 = None
        if (
            config_dict.get("COOLING_SIG_COL") is not None
            and config_dict.get("CLG_COIL_LEAVE_TEMP_COL") is not None
        ):
            fc14 = FaultConditionFourteen(config_dict)

        # Optionally initialize Fault Condition Fifteen
        fc15 = None
        if (
            config_dict.get("HTG_COIL_ENTER_TEMP_COL") is not None
            and config_dict.get("HTG_COIL_LEAVE_TEMP_COL") is not None
        ):
            fc15 = FaultConditionFifteen(config_dict)

        # Apply fault conditions and calculate fault counts
        df_fc1 = fc1.apply(df.copy())
        fault_counts["fc1_fault_sum"] = df_fc1["fc1_flag"].sum()

        df_fc2 = fc2.apply(df.copy())
        fault_counts["fc2_fault_sum"] = df_fc2["fc2_flag"].sum()

        df_fc3 = fc3.apply(df.copy())
        fault_counts["fc3_fault_sum"] = df_fc3["fc3_flag"].sum()

        df_fc4 = fc4.apply(df.copy())
        fault_counts["fc4_fault_sum"] = df_fc4["fc4_flag"].sum()

        df_fc5 = fc5.apply(df.copy())
        fault_counts["fc5_fault_sum"] = df_fc5["fc5_flag"].sum()

        if fc6 is not None:
            df_fc6 = fc6.apply(df.copy())
            fault_counts["fc6_fault_sum"] = df_fc6["fc6_flag"].sum()

        df_fc7 = fc7.apply(df.copy())
        fault_counts["fc7_fault_sum"] = df_fc7["fc7_flag"].sum()

        df_fc8 = fc8.apply(df.copy())
        fault_counts["fc8_fault_sum"] = df_fc8["fc8_flag"].sum()

        df_fc9 = fc9.apply(df.copy())
        fault_counts["fc9_fault_sum"] = df_fc9["fc9_flag"].sum()

        df_fc10 = fc10.apply(df.copy())
        fault_counts["fc10_fault_sum"] = df_fc10["fc10_flag"].sum()

        df_fc11 = fc11.apply(df.copy())
        fault_counts["fc11_fault_sum"] = df_fc11["fc11_flag"].sum()

        df_fc12 = fc12.apply(df.copy())
        fault_counts["fc12_fault_sum"] = df_fc12["fc12_flag"].sum()

        df_fc13 = fc13.apply(df.copy())
        fault_counts["fc13_fault_sum"] = df_fc13["fc13_flag"].sum()

        if fc14 is not None:
            df_fc14 = fc14.apply(df.copy())
            fault_counts["fc14_fault_sum"] = df_fc14["fc14_flag"].sum()

        if fc15 is not None:
            df_fc15 = fc15.apply(df.copy())
            fault_counts["fc15_fault_sum"] = df_fc15["fc15_flag"].sum()

        # Combine fault condition results
        df_combined = df_fc1.copy()
        df_combined["fc2_flag"] = df_fc2["fc2_flag"]
        df_combined["fc3_flag"] = df_fc3["fc3_flag"]
        df_combined["fc4_flag"] = df_fc4["fc4_flag"]
        df_combined["fc5_flag"] = df_fc5["fc5_flag"]

        if fc6 is not None:
            df_combined["fc6_flag"] = df_fc6["fc6_flag"]

        df_combined["fc7_flag"] = df_fc7["fc7_flag"]
        df_combined["fc8_flag"] = df_fc8["fc8_flag"]
        df_combined["fc9_flag"] = df_fc9["fc9_flag"]
        df_combined["fc10_flag"] = df_fc10["fc10_flag"]
        df_combined["fc11_flag"] = df_fc11["fc11_flag"]
        df_combined["fc12_flag"] = df_fc12["fc12_flag"]
        df_combined["fc13_flag"] = df_fc13["fc13_flag"]

        if fc14 is not None:
            df_combined["fc14_flag"] = df_fc14["fc14_flag"]

        if fc15 is not None:
            df_combined["fc15_flag"] = df_fc15["fc15_flag"]

        # Save fault counts to CSV
        fault_counts_df = pd.DataFrame(
            list(fault_counts.items()), columns=["Fault Condition", "Count"]
        )

        return df_combined, df_fc4, fault_counts_df



import pandas as pd
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
import sys


class FaultConditionOne(FaultCondition):
    """Class provides the definitions for Fault Condition 1.
    AHU low duct static pressure fan fault.
    """

    def __init__(self, dict_):
        """
        :param dict_:
        """
        self.vfd_speed_percent_err_thres = float
        self.vfd_speed_percent_max = float
        self.duct_static_inches_err_thres = float
        self.duct_static_col = str
        self.supply_vfd_speed_col = str
        self.duct_static_setpoint_col = str
        self.troubleshoot_mode = bool  # default should be False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # check analog outputs [data with units of %] are floats only
        columns_to_check = [self.supply_vfd_speed_col]
        self.check_analog_pct(df, columns_to_check)

        df["static_check_"] = (
            df[self.duct_static_col]
            < df[self.duct_static_setpoint_col] - self.duct_static_inches_err_thres
        )
        df["fan_check_"] = (
            df[self.supply_vfd_speed_col]
            >= self.vfd_speed_percent_max - self.vfd_speed_percent_err_thres
        )

        # Combined condition check
        df["combined_check"] = df["static_check_"] & df["fan_check_"]

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc1_flag"] = (rolling_sum == self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["static_check_"]
            del df["fan_check_"]
            del df["combined_check"]

        return df



import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import sys


class FaultCodeSixReport:
    """Class provides the definitions for Fault Condition 6 Report."""

    def __init__(self, config):
        self.supply_fan_air_volume_col = config["SUPPLY_FAN_AIR_VOLUME_COL"]
        self.mat_col = config["MAT_COL"]
        self.oat_col = config["OAT_COL"]
        self.rat_col = config["RAT_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc6_flag"

        fig, (ax1, ax2, ax3, ax4, ax5) = plt.subplots(5, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 6 Plot")

        ax1.plot(df.index, df["rat_minus_oat"], label="Rat Minus Oat")
        ax1.legend(loc="best")
        ax1.set_ylabel("°F")

        ax2.plot(
            df.index,
            df[self.supply_fan_air_volume_col],
            label="Total Air Flow",
            color="r",
        )
        ax2.set_xlabel("Date")
        ax2.set_ylabel("CFM")
        ax2.legend(loc="best")

        ax3.plot(df.index, df["percent_oa_calc"], label="OA Frac Calc", color="m")
        ax3.plot(df.index, df["perc_OAmin"], label="OA Perc Min Calc", color="y")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("%")
        ax3.legend(loc="best")

        ax4.plot(
            df.index,
            df["percent_oa_calc_minus_perc_OAmin"],
            label="OA Error Frac Vs Perc Min Calc",
            color="g",
        )
        ax4.set_xlabel("Date")
        ax4.set_ylabel("%")
        ax4.legend(loc="best")

        ax5.plot(df.index, df[output_col], label="Fault", color="k")
        ax5.set_xlabel("Date")
        ax5.set_ylabel("Fault Flags")
        ax5.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc6_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc6_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_rat": round(
                df[self.rat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_oat": round(
                df[self.oat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc6_flag"

        # Calculate dataset statistics
        df["hour_of_the_day_fc6"] = df.index.hour.where(df[output_col] == 1)

        # Make hist plots fc6
        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc6.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 6 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc6_flag"):
        # Display report content in IPython
        print(
            "Fault Condition 6: OA fraction too low or too high; should equal to design % outdoor air requirement"
        )

        # Display plot
        self.create_plot(df, output_col)

        # Display summary statistics
        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_mat = round(df[self.mat_col].where(df[output_col] == 1).mean(), 2)
            print("Mix Air Temp Mean When In Fault: ", flag_true_mat)

            flag_true_rat = round(df[self.rat_col].where(df[output_col] == 1).mean(), 2)
            print("Return Air Temp Mean When In Fault: ", flag_true_rat)

            flag_true_oat = round(df[self.oat_col].where(df[output_col] == 1).mean(), 2)
            print("Outside Air Temp Mean When In Fault: ", flag_true_oat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric maybe yielding sensors are out of calibration either on the AHU outside, mix, or return air temperature sensors that handle the OA fraction calculation or the totalized air flow calculation handled by a totalizing all VAV box air flows or AHU AFMS. Air flow and/or AHU temperature sensor may require recalibration."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low indicating the sensors are within calibration."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_eight import FaultConditionEight
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc8.py -rP -s

Supply air temperature approx equal mix temp in full econ.
"""

# Constants
TEST_DELTA_T_SUPPLY_FAN = 2.0
TEST_MIX_DEGF_ERR_THRES = 2.0
TEST_SUPPLY_DEGF_ERR_THRES = 2.0
TEST_AHU_MIN_OA_DPR = 0.2
TEST_MAT_COL = "mix_air_temp"
TEST_SAT_COL = "supply_air_temp"
TEST_ECONOMIZER_SIG_COL = "economizer_sig"
TEST_COOLING_SIG_COL = "cooling_sig"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionEight with a dictionary
fault_condition_params = {
    "DELTA_T_SUPPLY_FAN": TEST_DELTA_T_SUPPLY_FAN,
    "MIX_DEGF_ERR_THRES": TEST_MIX_DEGF_ERR_THRES,
    "SUPPLY_DEGF_ERR_THRES": TEST_SUPPLY_DEGF_ERR_THRES,
    "AHU_MIN_OA_DPR": TEST_AHU_MIN_OA_DPR,
    "MAT_COL": TEST_MAT_COL,
    "SAT_COL": TEST_SAT_COL,
    "ECONOMIZER_SIG_COL": TEST_ECONOMIZER_SIG_COL,
    "COOLING_SIG_COL": TEST_COOLING_SIG_COL,
    "TROUBLESHOOT_MODE": False,
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,
}

fc8 = FaultConditionEight(fault_condition_params)


class TestFaultConditionEight:

    def fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MAT_COL: [50, 50.5, 50.5, 50.8, 50.2, 50.6],
            TEST_SAT_COL: [40, 39.5, 39.8, 39.6, 39.2, 39.4],
            TEST_ECONOMIZER_SIG_COL: [0.5, 0.6, 0.5, 0.7, 0.5, 0.6],
            TEST_COOLING_SIG_COL: [0.05, 0.04, 0.03, 0.02, 0.01, 0.0],
        }
        return pd.DataFrame(data)

    def no_fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MAT_COL: [50, 50.5, 50.5, 50.8, 50.2, 50.6],
            TEST_SAT_COL: [51, 51.5, 50.9, 51.8, 51.2, 51.6],
            TEST_ECONOMIZER_SIG_COL: [0.5, 0.6, 0.5, 0.7, 0.5, 0.6],
            TEST_COOLING_SIG_COL: [0.05, 0.04, 0.03, 0.02, 0.01, 0.0],
        }
        return pd.DataFrame(data)

    def test_fault_condition_eight(self):
        results = fc8.apply(self.fault_df())
        actual = results["fc8_flag"].sum()
        expected = 2
        message = f"FC8 fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message

    def test_no_fault_condition_eight(self):
        results = fc8.apply(self.no_fault_df())
        actual = results["fc8_flag"].sum()
        expected = 0
        message = f"FC8 no_fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt:

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_MAT_COL: [50, 50.5, 50.5, 50.8, 50.2, 50.6],
            TEST_SAT_COL: [40, 39.5, 39.8, 39.6, 39.2, 39.4],
            TEST_ECONOMIZER_SIG_COL: [0.5, 0.6, 0.5, 0.7, 0.5, 0.6],
            TEST_COOLING_SIG_COL: [55, 56, 54, 55, 57, 55],  # Incorrect type
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_int_check_err(TEST_COOLING_SIG_COL)
        ):
            fc8.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne:

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_MAT_COL: [50, 50.5, 50.5, 50.8, 50.2, 50.6],
            TEST_SAT_COL: [40, 39.5, 39.8, 39.6, 39.2, 39.4],
            TEST_ECONOMIZER_SIG_COL: [0.5, 0.6, 0.5, 0.7, 0.5, 0.6],
            TEST_COOLING_SIG_COL: [1.1, 1.2, 1.1, 1.3, 1.1, 1.2],  # Values > 1.0
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_max_check_err(TEST_COOLING_SIG_COL)
        ):
            fc8.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes:

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_MAT_COL: [50, 50.5, 50.5, 50.8, 50.2, 50.6],
            TEST_SAT_COL: [40, 39.5, 39.8, 39.6, 39.2, 39.4],
            TEST_ECONOMIZER_SIG_COL: [0.5, 0.6, 0.5, 0.7, 0.5, 0.6],
            TEST_COOLING_SIG_COL: [1.1, 0.55, 1.2, 1.3, 0.55, 1.1],  # Mixed types
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_max_check_err(TEST_COOLING_SIG_COL)
        ):
            fc8.apply(self.fault_df_on_mixed_types())


if __name__ == "__main__":
    pytest.main()



import pandas as pd
import numpy as np
import operator
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionTwelve(FaultCondition):
    """Class provides the definitions for Fault Condition 12.
    Supply air temperature too high; should be less than
    mix air temperature in economizer plus mech cooling mode.
    """

    def __init__(self, dict_):
        self.delta_t_supply_fan = float
        self.mix_degf_err_thres = float
        self.supply_degf_err_thres = float
        self.ahu_min_oa_dpr = float
        self.sat_col = str
        self.mat_col = str
        self.cooling_sig_col = str
        self.economizer_sig_col = str
        self.troubleshoot_mode = bool  # default False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # Check analog outputs [data with units of %] are floats only
        columns_to_check = [
            self.economizer_sig_col,
            self.cooling_sig_col,
        ]
        self.check_analog_pct(df, columns_to_check)

        # Create helper columns
        df["sat_minus_saterr_delta_supply_fan"] = (
            df[self.sat_col] - self.supply_degf_err_thres - self.delta_t_supply_fan
        )
        df["mat_plus_materr"] = df[self.mat_col] + self.mix_degf_err_thres

        df["combined_check"] = operator.or_(
            # OS4 AHU state clg @ min OA
            (df["sat_minus_saterr_delta_supply_fan"] > df["mat_plus_materr"])
            # verify AHU in OS4 mode
            & (df[self.cooling_sig_col] > 0.01)
            & (df[self.economizer_sig_col] == self.ahu_min_oa_dpr),  # OR
            (df["sat_minus_saterr_delta_supply_fan"] > df["mat_plus_materr"])
            # verify ahu is running in OS 3 clg mode in 100 OA
            & (df[self.cooling_sig_col] > 0.01) & (df[self.economizer_sig_col] > 0.9),
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc12_flag"] = (rolling_sum >= self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["sat_minus_saterr_delta_supply_fan"]
            del df["mat_plus_materr"]
            del df["combined_check"]

        return df



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_one import FaultConditionOne
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

# Constants
TEST_VFD_ERR_THRESHOLD = 0.05
TEST_VFD_SPEED_MAX = 0.7
TEST_DUCT_STATIC_ERR_THRESHOLD = 0.1
TEST_DUCT_STATIC_COL = "duct_static"
TEST_DUCT_STATIC_SETPOINT_COL = "duct_static_setpoint"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionOne with a dictionary
fault_condition_params = {
    "VFD_SPEED_PERCENT_ERR_THRES": TEST_VFD_ERR_THRESHOLD,
    "VFD_SPEED_PERCENT_MAX": TEST_VFD_SPEED_MAX,
    "DUCT_STATIC_INCHES_ERR_THRES": TEST_DUCT_STATIC_ERR_THRESHOLD,
    "DUCT_STATIC_COL": TEST_DUCT_STATIC_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "DUCT_STATIC_SETPOINT_COL": TEST_DUCT_STATIC_SETPOINT_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,  # rolling sum window size
}

fc1 = FaultConditionOne(fault_condition_params)


class TestNoFault:

    def no_fault_df(self) -> pd.DataFrame:
        data = {
            TEST_DUCT_STATIC_COL: [0.99, 0.99, 0.99, 0.99, 0.99, 0.99],
            TEST_DUCT_STATIC_SETPOINT_COL: [1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.8, 0.8, 0.8, 0.8, 0.8, 0.8],
        }
        return pd.DataFrame(data)

    def test_no_fault(self):
        results = fc1.apply(self.no_fault_df())
        actual = results["fc1_flag"].sum()
        expected = 0
        message = f"FC1 no_fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFault:

    def fault_df(self) -> pd.DataFrame:
        data = {
            TEST_DUCT_STATIC_COL: [
                0.7,
                0.7,
                0.6,
                0.7,
                0.65,
                0.55,
                0.99,
                0.99,
                0.6,
                0.7,
                0.65,
                0.55,
                0.6,
                0.7,
                0.65,
                0.55,
                0.6,
            ],
            TEST_DUCT_STATIC_SETPOINT_COL: [
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
                1.0,
            ],
            TEST_SUPPLY_VFD_SPEED_COL: [
                0.99,
                0.95,
                0.96,
                0.97,
                0.98,
                0.98,
                0.5,
                0.55,
                0.96,
                0.97,
                0.98,
                0.98,
                0.96,
                0.97,
                0.98,
                0.98,
                0.96,
            ],
        }
        return pd.DataFrame(data)

    def test_fault(self):
        results = fc1.apply(self.fault_df())
        actual = results["fc1_flag"].sum()

        # accumilated 5 faults need to happen before an "official fault"
        # in TEST_DUCT_STATIC_COL after the 5 first values there is 3 faults
        # then artificially adjust fake fan data back to normal and another 5
        # needs happen per ROLLING_WINDOW_SIZE and then 4 faults after that.
        # so expected = 3 + 4.
        expected = 3 + 4
        message = f"FC1 fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt:

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_DUCT_STATIC_COL: [0.8] * 6,
            TEST_DUCT_STATIC_SETPOINT_COL: [1.0] * 6,
            TEST_SUPPLY_VFD_SPEED_COL: [99] * 6,
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc1.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne:

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_DUCT_STATIC_COL: [0.8] * 6,
            TEST_DUCT_STATIC_SETPOINT_COL: [1.0] * 6,
            TEST_SUPPLY_VFD_SPEED_COL: [99.0] * 6,
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc1.apply(self.fault_df_on_output_greater_than_one())



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeFourteenReport:
    """Class provides the definitions for Fault Condition 14 Report."""

    def __init__(self, config):
        self.sat_col = config["SAT_COL"]
        self.clt_col = config["CLT_COL"]
        self.cooling_sig_col = config["COOLING_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc14_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 14 Plot")

        ax1.plot(df.index, df[self.sat_col], label="SAT")
        ax1.plot(df.index, df[self.clt_col], label="CLT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[self.cooling_sig_col], label="AHU Cool Vlv", color="r")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc14_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc14_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_clt": round(
                df[self.clt_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat": round(
                df[self.sat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc14_flag"

        df["hour_of_the_day_fc14"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc14.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 14 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(
        self, df: pd.DataFrame, output_col: str = "fc14_flag"
    ):
        print(
            "Fault Condition 14: Temperature drop across inactive cooling coil (requires coil leaving temp sensor)"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_clt = round(df[self.clt_col].where(df[output_col] == 1).mean(), 2)
            print("Coil Leaving Temp Mean When In Fault: ", flag_true_clt)

            flag_true_sat = round(df[self.sat_col].where(df[output_col] == 1).mean(), 2)
            print("Supply Air Temp Mean When In Fault: ", flag_true_sat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high, indicating potential leakage in the cooling coil. Verify temperature sensor calibration and investigate possible mechanical issues."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the cooling coil is likely functioning correctly."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionFour(FaultCondition):
    """Class provides the definitions for Fault Condition 4.

    This fault flags excessive operating states on the AHU
    if its hunting between heating, econ, econ+mech, and
    a mech clg modes. The code counts how many operating
    changes in an hour and will throw a fault if there is
    excessive OS changes to flag control sys hunting.

    """

    def __init__(self, dict_):
        self.delta_os_max = float
        self.ahu_min_oa_dpr = float
        self.economizer_sig_col = str
        self.heating_sig_col = str
        self.cooling_sig_col = str
        self.supply_vfd_speed_col = str
        self.troubleshoot_mode = bool  # default to False

        self.set_attributes(dict_)

    # adds in these boolean columns to the dataframe
    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # check analog outputs [data with units of %] are floats only
        columns_to_check = [
            self.economizer_sig_col,
            self.heating_sig_col,
            self.cooling_sig_col,
            self.supply_vfd_speed_col,
        ]

        for col in columns_to_check:
            self.check_analog_pct(df, [col])

        print("=" * 50)
        print("Warning: The program is in FC4 and resampling the data")
        print("to compute AHU OS state changes per hour")
        print("to flag any hunting issue")
        print("and this usually takes a while to run...")
        print("=" * 50)

        sys.stdout.flush()

        # AHU htg only mode based on OA damper @ min oa and only htg pid/vlv modulating
        df["heating_mode"] = (
            (df[self.heating_sig_col] > 0)
            & (df[self.cooling_sig_col] == 0)
            & (df[self.supply_vfd_speed_col] > 0)
            & (df[self.economizer_sig_col] == self.ahu_min_oa_dpr)
        )

        # AHU econ only mode based on OA damper modulating and clg htg = zero
        df["econ_only_cooling_mode"] = (
            (df[self.heating_sig_col] == 0)
            & (df[self.cooling_sig_col] == 0)
            & (df[self.supply_vfd_speed_col] > 0)
            & (df[self.economizer_sig_col] > self.ahu_min_oa_dpr)
        )

        # AHU econ+mech clg mode based on OA damper modulating for cooling and clg pid/vlv modulating
        df["econ_plus_mech_cooling_mode"] = (
            (df[self.heating_sig_col] == 0)
            & (df[self.cooling_sig_col] > 0)
            & (df[self.supply_vfd_speed_col] > 0)
            & (df[self.economizer_sig_col] > self.ahu_min_oa_dpr)
        )

        # AHU mech mode based on OA damper @ min OA and clg pid/vlv modulating
        df["mech_cooling_only_mode"] = (
            (df[self.heating_sig_col] == 0)
            & (df[self.cooling_sig_col] > 0)
            & (df[self.supply_vfd_speed_col] > 0)
            & (df[self.economizer_sig_col] == self.ahu_min_oa_dpr)
        )

        # Fill non-finite values with zero or drop them
        df = df.fillna(0)

        df = df.astype(int)
        df = df.resample("60min").apply(lambda x: (x.eq(1) & x.shift().ne(1)).sum())

        df["fc4_flag"] = df[df.columns].gt(self.delta_os_max).any(axis=1).astype(int)
        return df



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_fourteen import (
    FaultConditionFourteen,
)
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest open_fdd/tests/ahu/test_ahu_fc14.py -rP -s

Temp drop across inactive clg coil in OS1 & OS2
"""

# Constants
TEST_DELTA_SUPPLY_FAN = 2.0
TEST_COIL_TEMP_ENTER_ERR_THRES = 1.0
TEST_COIL_TEMP_LEAVE_ERR_THRES = 1.0
TEST_AHU_MIN_OA_DPR = 0.2
TEST_CLG_COIL_ENTER_TEMP_COL = "clg_enter_air_temp"
TEST_CLG_COIL_LEAVE_TEMP_COL = "clg_leave_air_temp"
TEST_CLG_COIL_CMD_COL = "cooling_sig_col"
TEST_HTG_COIL_CMD_COL = "heating_sig_col"
TEST_MIX_AIR_DAMPER_COL = "economizer_sig_col"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionFourteen with a dictionary
fault_condition_params = {
    "DELTA_T_SUPPLY_FAN": TEST_DELTA_SUPPLY_FAN,
    "COIL_TEMP_ENTER_ERR_THRES": TEST_COIL_TEMP_ENTER_ERR_THRES,
    "COIL_TEMP_LEAV_ERR_THRES": TEST_COIL_TEMP_LEAVE_ERR_THRES,
    "AHU_MIN_OA_DPR": TEST_AHU_MIN_OA_DPR,
    "CLG_COIL_ENTER_TEMP_COL": TEST_CLG_COIL_ENTER_TEMP_COL,
    "CLG_COIL_LEAVE_TEMP_COL": TEST_CLG_COIL_LEAVE_TEMP_COL,
    "COOLING_SIG_COL": TEST_CLG_COIL_CMD_COL,
    "HEATING_SIG_COL": TEST_HTG_COIL_CMD_COL,
    "ECONOMIZER_SIG_COL": TEST_MIX_AIR_DAMPER_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,
}

fc14 = FaultConditionFourteen(fault_condition_params)


class TestFaultConditionFourteen:

    def no_fault_df_econ(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [56.5, 56.5, 56.5, 56.5, 56.5, 56.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_MIX_AIR_DAMPER_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def no_fault_df_htg(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [56.5, 56.5, 56.5, 56.5, 56.5, 56.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_MIX_AIR_DAMPER_COL: [
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
            ],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def fault_df_in_econ(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [50.5, 50.5, 50.5, 50.5, 50.5, 50.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_MIX_AIR_DAMPER_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def fault_df_in_htg(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [50.5, 50.5, 50.5, 50.5, 50.5, 50.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_MIX_AIR_DAMPER_COL: [
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
                TEST_AHU_MIN_OA_DPR,
            ],
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def test_no_fault_econ(self):
        results = fc14.apply(self.no_fault_df_econ())
        actual = results["fc14_flag"].sum()
        expected = 0
        message = f"FC14 no_fault_df_econ actual is {actual} and expected is {expected}"
        assert actual == expected, message

    def test_no_fault_htg(self):
        results = fc14.apply(self.no_fault_df_htg())
        actual = results["fc14_flag"].sum()
        expected = 0
        message = f"FC14 no_fault_df_htg actual is {actual} and expected is {expected}"
        assert actual == expected, message

    def test_fault_in_econ(self):
        results = fc14.apply(self.fault_df_in_econ())
        actual = results["fc14_flag"].sum()
        expected = 2
        message = f"FC14 fault_df_in_econ actual is {actual} and expected is {expected}"
        assert actual == expected, message

    def test_fault_in_htg(self):
        results = fc14.apply(self.fault_df_in_htg())
        actual = results["fc14_flag"].sum()
        expected = 2
        message = f"FC14 fault_df_in_htg actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt:

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [50.5, 50.5, 50.5, 50.5, 50.5, 50.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_MIX_AIR_DAMPER_COL: [55, 55, 55, 55, 55, 55],  # Incorrect type
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_int_check_err(TEST_MIX_AIR_DAMPER_COL)
        ):
            fc14.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne:

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [50.5, 50.5, 50.5, 50.5, 50.5, 50.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_MIX_AIR_DAMPER_COL: [1.1, 1.2, 1.1, 1.3, 1.1, 1.2],  # Values > 1.0
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_max_check_err(TEST_MIX_AIR_DAMPER_COL)
        ):
            fc14.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes:

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_CLG_COIL_ENTER_TEMP_COL: [55, 55, 55, 55, 55, 55],
            TEST_CLG_COIL_LEAVE_TEMP_COL: [50.5, 50.5, 50.5, 50.5, 50.5, 50.5],
            TEST_CLG_COIL_CMD_COL: [0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            TEST_HTG_COIL_CMD_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
            TEST_MIX_AIR_DAMPER_COL: [1.1, 0.55, 1.2, 1.3, 0.55, 1.1],  # Mixed types
            TEST_SUPPLY_VFD_SPEED_COL: [0.55, 0.55, 0.55, 0.55, 0.55, 0.55],
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError, match=HelperUtils().float_max_check_err(TEST_MIX_AIR_DAMPER_COL)
        ):
            fc14.apply(self.fault_df_on_mixed_types())


if __name__ == "__main__":
    pytest.main()



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeThirteenReport:
    """Class provides the definitions for Fault Condition 13 Report."""

    def __init__(self, config):
        self.sat_col = config["SAT_COL"]
        self.mat_col = config["MAT_COL"]
        self.cooling_sig_col = config["COOLING_SIG_COL"]
        self.economizer_sig_col = config["ECONOMIZER_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc13_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 13 Plot")

        ax1.plot(df.index, df[self.sat_col], label="SAT")
        ax1.plot(df.index, df[self.mat_col], label="MAT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[self.cooling_sig_col], label="AHU Cool Vlv", color="r")
        ax2.plot(df.index, df[self.economizer_sig_col], label="AHU Dpr Cmd", color="g")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc13_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc13_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat": round(
                df[self.sat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc13_flag"

        df["hour_of_the_day_fc13"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc13.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 13 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(
        self, df: pd.DataFrame, output_col: str = "fc13_flag"
    ):
        print(
            "Fault Condition 13: Supply air temperature too high in full cooling in economizer plus mech cooling mode"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_mat = round(df[self.mat_col].where(df[output_col] == 1).mean(), 2)
            print("Mix Air Temp Mean When In Fault: ", flag_true_mat)

            flag_true_sat = round(df[self.sat_col].where(df[output_col] == 1).mean(), 2)
            print("Supply Air Temp Mean When In Fault: ", flag_true_sat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high, indicating temperature sensor error or issues with the cooling system. Verify AHU temperature sensor calibration and investigate potential mechanical issues."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the AHU components are within calibration for this fault equation."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import pytest
from open_fdd.air_handling_unit.faults.fault_condition_three import FaultConditionThree
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils

"""
To see print statements in pytest run with:
$ py -3.12 -m pytest tests/ahu/test_ahu_fc3.py -rP -s

Mix air temp higher than out temp
"""

# Constants
TEST_MIX_DEGF_ERR_THRES = 2.0
TEST_RETURN_DEGF_ERR_THRES = 2.0
TEST_OUTDOOR_DEGF_ERR_THRES = 5.0
TEST_MIX_TEMP_COL = "mix_air_temp"
TEST_RETURN_TEMP_COL = "return_air_temp"
TEST_OUT_TEMP_COL = "out_air_temp"
TEST_SUPPLY_VFD_SPEED_COL = "supply_vfd_speed"
ROLLING_WINDOW_SIZE = 5

# Initialize FaultConditionThree with a dictionary
fault_condition_params = {
    "MIX_DEGF_ERR_THRES": TEST_MIX_DEGF_ERR_THRES,
    "RETURN_DEGF_ERR_THRES": TEST_RETURN_DEGF_ERR_THRES,
    "OUTDOOR_DEGF_ERR_THRES": TEST_OUTDOOR_DEGF_ERR_THRES,
    "MAT_COL": TEST_MIX_TEMP_COL,
    "RAT_COL": TEST_RETURN_TEMP_COL,
    "OAT_COL": TEST_OUT_TEMP_COL,
    "SUPPLY_VFD_SPEED_COL": TEST_SUPPLY_VFD_SPEED_COL,
    "TROUBLESHOOT_MODE": False,  # default value
    "ROLLING_WINDOW_SIZE": ROLLING_WINDOW_SIZE,
}

fc3 = FaultConditionThree(fault_condition_params)


class TestNoFault:

    def no_fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [55.0, 56.0, 57.0, 56.0, 55.5, 55.0],
            TEST_RETURN_TEMP_COL: [70.0, 71.0, 72.0, 70.0, 71.0, 70.0],
            TEST_OUT_TEMP_COL: [50.0, 51.0, 52.0, 50.0, 51.0, 50.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.8, 0.82, 0.83, 0.8, 0.82, 0.8],
        }
        return pd.DataFrame(data)

    def test_no_fault(self):
        results = fc3.apply(self.no_fault_df())
        actual = results["fc3_flag"].sum()
        expected = 0
        message = f"FC3 no_fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFault:

    def fault_df(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_RETURN_TEMP_COL: [70.0, 70.5, 71.0, 70.0, 70.5, 70.0],
            TEST_OUT_TEMP_COL: [50.0, 51.0, 52.0, 50.5, 51.0, 50.0],
            TEST_SUPPLY_VFD_SPEED_COL: [0.9, 0.91, 0.92, 0.9, 0.91, 0.9],
        }
        return pd.DataFrame(data)

    def test_fault(self):
        results = fc3.apply(self.fault_df())
        actual = results["fc3_flag"].sum()
        expected = 2
        message = f"FC3 fault_df actual is {actual} and expected is {expected}"
        assert actual == expected, message


class TestFaultOnInt:

    def fault_df_on_output_int(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_RETURN_TEMP_COL: [70.0, 70.5, 71.0, 70.0, 70.5, 70.0],
            TEST_OUT_TEMP_COL: [50.0, 51.0, 52.0, 50.5, 51.0, 50.0],
            TEST_SUPPLY_VFD_SPEED_COL: [99, 99, 99, 99, 99, 99],
        }
        return pd.DataFrame(data)

    def test_fault_on_int(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_int_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc3.apply(self.fault_df_on_output_int())


class TestFaultOnFloatGreaterThanOne:

    def fault_df_on_output_greater_than_one(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_RETURN_TEMP_COL: [70.0, 70.5, 71.0, 70.0, 70.5, 70.0],
            TEST_OUT_TEMP_COL: [50.0, 51.0, 52.0, 50.5, 51.0, 50.0],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 1.2, 1.3, 1.1, 1.2, 1.1],
        }
        return pd.DataFrame(data)

    def test_fault_on_float_greater_than_one(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc3.apply(self.fault_df_on_output_greater_than_one())


class TestFaultOnMixedTypes:

    def fault_df_on_mixed_types(self) -> pd.DataFrame:
        data = {
            TEST_MIX_TEMP_COL: [80.0, 81.0, 79.0, 80.5, 82.0, 80.0],
            TEST_RETURN_TEMP_COL: [70.0, 70.5, 71.0, 70.0, 70.5, 70.0],
            TEST_OUT_TEMP_COL: [50.0, 51.0, 52.0, 50.5, 51.0, 50.0],
            TEST_SUPPLY_VFD_SPEED_COL: [1.1, 0.91, 1.3, 1.1, 0.92, 1.1],
        }
        return pd.DataFrame(data)

    def test_fault_on_mixed_types(self):
        with pytest.raises(
            TypeError,
            match=HelperUtils().float_max_check_err(TEST_SUPPLY_VFD_SPEED_COL),
        ):
            fc3.apply(self.fault_df_on_mixed_types())



import pandas as pd
import numpy as np
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionSeven(FaultCondition):
    """Class provides the definitions for Fault Condition 7.
    Very similar to FC 13 but uses heating valve.
    Supply air temperature too low in full heating.
    """

    def __init__(self, dict_):
        self.supply_degf_err_thres = float
        self.sat_col = str
        self.sat_setpoint_col = str
        self.heating_sig_col = str
        self.supply_vfd_speed_col = str
        self.troubleshoot_mode = bool  # default to False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # Check analog outputs [data with units of %] are floats only
        columns_to_check = [self.supply_vfd_speed_col, self.heating_sig_col]
        self.check_analog_pct(df, columns_to_check)

        # Fault condition-specific checks / flags
        df["sat_check"] = df[self.sat_setpoint_col] - self.supply_degf_err_thres

        df["combined_check"] = (
            (df[self.sat_col] < df["sat_check"])
            & (df[self.heating_sig_col] > 0.9)
            & (df[self.supply_vfd_speed_col] > 0)
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc7_flag"] = (rolling_sum >= self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["sat_check"]
            del df["combined_check"]

        return df



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeEightReport:
    """Class provides the definitions for Fault Condition 8 Report."""

    def __init__(self, config):
        self.sat_col = config["SAT_COL"]
        self.mat_col = config["MAT_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]
        self.economizer_sig_col = config["ECONOMIZER_SIG_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc8_flag"

        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 8 Plot")

        ax1.plot(df.index, df[self.sat_col], label="SAT")
        ax1.plot(df.index, df[self.mat_col], label="MAT")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Temps °F")

        ax2.plot(df.index, df[output_col], label="Fault", color="k")
        ax2.set_xlabel("Date")
        ax2.set_ylabel("Fault Flags")
        ax2.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc8_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc8_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_mat": round(
                df[self.mat_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat": round(
                df[self.sat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc8_flag"

        df["hour_of_the_day_fc8"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc8.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 8 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc8_flag"):
        print(
            "Fault Condition 8: Supply air temperature and mix air temperature should be approximately equal in economizer mode"
        )

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_mat = round(df[self.mat_col].where(df[output_col] == 1).mean(), 2)
            print("Mix Air Temp Mean When In Fault: ", flag_true_mat)

            flag_true_sat = round(df[self.sat_col].where(df[output_col] == 1).mean(), 2)
            print("Supply Air Temp Mean When In Fault: ", flag_true_sat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high indicating temperature sensor error or the heating/cooling coils are leaking, potentially creating simultaneous heating/cooling which can be an energy penalty for running the AHU in this fashion. Verify AHU mix/supply temperature sensor calibration in addition to a potential mechanical issue of a leaking valve. A leaking valve can be troubleshot by isolating the valve closed by manual shut-off valves where piping lines enter the AHU coil and then verifying any changes in the AHU discharge air temperature."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the AHU components are within calibration for this fault equation."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import matplotlib.pyplot as plt
import pandas as pd
import sys


class FaultCodeSevenReport:
    """Class provides the definitions for Fault Condition 7 Report.
    Very similar to FC 13 but uses heating valve
    """

    def __init__(self, config):
        self.sat_col = config["SAT_COL"]
        self.sat_setpoint_col = config["SAT_SETPOINT_COL"]
        self.heating_sig_col = config["HEATING_SIG_COL"]
        self.supply_vfd_speed_col = config["SUPPLY_VFD_SPEED_COL"]

    def create_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc7_flag"

        fig, (ax1, ax2, ax3) = plt.subplots(3, 1, figsize=(25, 8))
        fig.suptitle("Fault Conditions 7 Plot")

        ax1.plot(df.index, df[self.sat_col], label="SAT")
        ax1.plot(df.index, df[self.sat_setpoint_col], label="SATsp")
        ax1.legend(loc="best")
        ax1.set_ylabel("AHU Supply Temps °F")

        ax2.plot(df.index, df[self.heating_sig_col], color="r", label="AHU Heat Vlv")
        ax2.legend(loc="best")
        ax2.set_ylabel("%")

        ax3.plot(df.index, df[output_col], label="Fault", color="k")
        ax3.set_xlabel("Date")
        ax3.set_ylabel("Fault Flags")
        ax3.legend(loc="best")

        plt.tight_layout(rect=[0, 0.03, 1, 0.95])
        plt.show()
        plt.close()

    def summarize_fault_times(self, df: pd.DataFrame, output_col: str = None) -> dict:
        if output_col is None:
            output_col = "fc7_flag"

        delta = df.index.to_series().diff()
        summary = {
            "total_days": round(delta.sum() / pd.Timedelta(days=1), 2),
            "total_hours": round(delta.sum() / pd.Timedelta(hours=1)),
            "hours_fc7_mode": round(
                (delta * df[output_col]).sum() / pd.Timedelta(hours=1)
            ),
            "percent_true": round(df[output_col].mean() * 100, 2),
            "percent_false": round((100 - round(df[output_col].mean() * 100, 2)), 2),
            "flag_true_satsp": round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            ),
            "flag_true_sat": round(
                df[self.sat_col].where(df[output_col] == 1).mean(), 2
            ),
            "hours_motor_runtime": round(
                (delta * df[self.supply_vfd_speed_col].gt(0.01).astype(int)).sum()
                / pd.Timedelta(hours=1),
                2,
            ),
        }

        return summary

    def create_hist_plot(self, df: pd.DataFrame, output_col: str = None):
        if output_col is None:
            output_col = "fc7_flag"

        df["hour_of_the_day_fc7"] = df.index.hour.where(df[output_col] == 1)

        fig, ax = plt.subplots(tight_layout=True, figsize=(25, 8))
        ax.hist(df.hour_of_the_day_fc7.dropna())
        ax.set_xlabel("24 Hour Number in Day")
        ax.set_ylabel("Frequency")
        ax.set_title(f"Hour-Of-Day When Fault Flag 7 is TRUE")
        plt.show()
        plt.close()

    def display_report_in_ipython(self, df: pd.DataFrame, output_col: str = "fc7_flag"):
        print("Fault Condition 7: Supply air temperature too low in full heating mode")

        self.create_plot(df, output_col)

        summary = self.summarize_fault_times(df, output_col)

        for key, value in summary.items():
            formatted_key = key.replace("_", " ")
            print(f"{formatted_key}: {value}")
            sys.stdout.flush()

        fc_max_faults_found = df[output_col].max()
        print("Fault Flag Count: ", fc_max_faults_found)
        sys.stdout.flush()

        if fc_max_faults_found != 0:
            self.create_hist_plot(df, output_col)

            flag_true_satsp = round(
                df[self.sat_setpoint_col].where(df[output_col] == 1).mean(), 2
            )
            print("Supply Air Temp Setpoint Mean When In Fault: ", flag_true_satsp)

            flag_true_sat = round(df[self.sat_col].where(df[output_col] == 1).mean(), 2)
            print("Supply Air Temp Mean When In Fault: ", flag_true_sat)

            sys.stdout.flush()

            if summary["percent_true"] > 5.0:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is high indicating the AHU heating valve may be broken or there could be a flow issue with the amount of hot water flowing through the coil or that the boiler system reset is too aggressive and there isn’t enough heat being produced by this coil. It could be worth viewing mechanical blueprints for this AHU design schedule to see what hot water temperature this coil was designed for and compare it to actual hot water supply temperatures. Consult a mechanical design engineer to rectify if needed."
                )
            else:
                print(
                    "The percent True metric that represents the amount of time for when the fault flag is True is low, indicating the AHU heating valve operates correctly."
                )

        else:
            print("NO FAULTS FOUND - Skipping time-of-day Histogram plot")
            sys.stdout.flush()



import pandas as pd
import numpy as np
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultConditionTen(FaultCondition):
    """Class provides the definitions for Fault Condition 10.
    Outdoor air temperature and mix air temperature should
    be approx equal in economizer plus mech cooling mode.
    """

    def __init__(self, dict_):
        self.outdoor_degf_err_thres = float
        self.mix_degf_err_thres = float
        self.oat_col = str
        self.mat_col = str
        self.cooling_sig_col = str
        self.economizer_sig_col = str
        self.troubleshoot_mode = bool  # default False,
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # Check analog outputs [data with units of %] are floats only
        columns_to_check = [
            self.economizer_sig_col,
            self.cooling_sig_col,
        ]
        self.check_analog_pct(df, columns_to_check)

        df["abs_mat_minus_oat"] = abs(df[self.mat_col] - df[self.oat_col])
        df["mat_oat_sqrted"] = np.sqrt(
            self.mix_degf_err_thres**2 + self.outdoor_degf_err_thres**2
        )

        df["combined_check"] = (
            (df["abs_mat_minus_oat"] > df["mat_oat_sqrted"])
            # verify ahu is running in OS 3 clg mode in min OA
            & (df[self.cooling_sig_col] > 0.01)
            & (df[self.economizer_sig_col] > 0.9)
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc10_flag"] = (rolling_sum >= self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            del df["abs_mat_minus_oat"]
            del df["mat_oat_sqrted"]
            del df["combined_check"]

        return df



import streamlit as st


# On windows 10 python 3.10.6
# py -3.10 -m streamlit run .\latex_generator.py

# display Fault Equation 1
st.title("Fault Equation 1")
st.caption("Duct static pressure too low with fan at full speed")
st.latex(
    r"""
    DSP < DPSP - eDSP \quad \text{and} \quad VFDSPD \geq 99\% - eVFDSPD
    """
)

# Display legend
st.markdown("Legend:")
st.markdown("- DSP: Duct Static Pressure")
st.markdown("- DPSP: Duct Static Pressure Setpoint")
st.markdown("- VFDSPD: VFD Speed Reference in Percent")
st.markdown("- eVFDSPD: VFD Speed Reference Error Threshold")


# display Fault Equation 2
st.title("Fault Equation 2")
st.caption("Mix air temperature too low; should be between outside and return")
st.latex(
    r"""
    MAT_{avg} + eMAT < \min[(RAT_{avg} - eRAT), - OAT_{avg} - eOAT)]
    """
)


# display Fault Equation 3
st.title("Fault Equation 3")
st.caption("Mix air temperature too high; should be between outside and return")
st.latex(
    r"""
    MAT_{avg} - eMAT > \min[(RAT_{avg} + eRAT), - OAT_{avg} + eOAT)]
    """
)


# display Fault Equation 4
st.title("Fault Equation 4")
st.caption(
    "Too many AHU operating state changes due to PID hunting and/or excessive cycling during low load conditions."
)
st.latex(
    r"""
    \Delta OS > \Delta OS_{max}
    """
)


# display Fault Equation 5
st.title("Fault Equation 5")
st.caption("Supply air temperature too high")
st.latex(
    r"""
    SAT_{avg} + eSAT \leq MAT_{avg} - eMAT + \Delta TSF
    """
)


# display Fault Equation 6
st.title("Fault Equation 6")
st.caption("Temperature and outside air percentage deviation from setpoints")
st.latex(
    r"""
    |\text{RAT}_{\text{avg}} - \text{OAT}_{\text{avg}}| \geq \Delta T_{\text{min}} \quad \text{and} \quad |\%OA - \%OA_{\text{min}}| > eF
    """
)


# display Fault Equation 7
st.title("Fault Equation 7")
st.caption("Supply air temperature too low and heating coil status")
st.latex(
    r"""
    \text{SAT}_{\text{avg}} < \text{SATSP} - eSAT \quad \text{and} \quad \text{HC} \geq 99\%
    """
)

# display Fault Equation 8
st.title("Fault Equation 8")
st.caption("Deviation between supply air temperature and mixed air temperature")
st.latex(
    r"""
    | \text{SAT}_{\text{avg}} - \Delta \text{TSF} - \text{MAT}_{\text{avg}} | > \sqrt{{eSAT}^2 + {eMAT}^2}
    """
)


# display Fault Equation 9
st.title("Fault Equation 9")
st.caption("Outside air temperature deviation from setpoint")
st.latex(
    r"""
    \text{OAT}_{\text{avg}} - eOAT > \text{SATSP} - \Delta \text{SF} + eSAT
    """
)

# display Fault Equation 10
st.title("Fault Equation 10")
st.caption("Temperature difference between mixed air and outside air")
st.latex(
    r"""
    | \text{MAT}_{\text{avg}} - \text{OAT}_{\text{avg}} | > \sqrt{eMAT^2 + eOAT^2}
    """
)

# display Fault Equation 11
st.title("Fault Equation 11")
st.caption("Outside air temperature and supply air temperature deviation")
st.latex(
    r"""
    \text{OAT}_{\text{avg}} + eOAT < \text{SATSP} - \Delta \text{TSF} - eSAT
    """
)

# display Fault Equation 12
st.title("Fault Equation 12")
st.caption("Supply air temperature deviation from mixed air temperature")
st.latex(
    r"""
    \text{SAT}_{\text{avg}} - eSAT - \Delta \text{TSF} \geq \text{MAT}_{\text{avg}} + eMAT
    """
)

# display Fault Equation 13
st.title("Fault Equation 13")
st.caption("Supply air temperature too high")
st.latex(
    r"""
    \text{SAT}_{\text{avg}} < \text{SATSP} + eSAT \quad \text{and} \quad \text{CC} \geq 99\%
    """
)


st.title("find_closest_weather_dates Function")
st.caption("Finding closest weather dates based on given criteria")
st.latex(
    r"""
1. A' = \{a \in A : a < d_{test}\} \\
2. B' = \{b \in B : b < d_{test}\} \\
3. C = \{a \in A' : a \notin B'\} \\
4. \text{if } |C| < 10 \text{ then remove } \max(A') \text{ and repeat step 3} \\
5. A = A \cap C, \text{calculate } \mu(A)
"""
)
st.caption(
    """
In this notation:
- $A$ represents the "all_data" dataset.
- $B$ represents the "suitable_baseline_no" dataset.
- $d_{test}$ is the "test_case_date".
- $A'$ and $B'$ are subsets of $A$ and $B$ that only include dates prior to $d_{test}$.
- $C$ is a set of dates in $A'$ not found in $B'$.
- $|C|$ represents the count of elements in set $C$.
- $\max(A')$ is the latest date in $A'$.
- $\mu(A)$ is the mean of the remaining elements in $A$ after filtering by set $C$.
"""
)


st.title("find_previous_10_days Function")
st.caption("Finding previous 10 weekdays based on given criteria")
st.latex(
    r"""
1. A' = \{a \in A : a < d_{test}\} \\
2. B' = \{b \in B : b < d_{test}\} \\
3. C = \{a \in A' : a \notin B'\} \\
4. \text{if } |C| < 10 \text{ then remove } \max(A') \text{ and repeat step 3} \\
5. A = A \cap C
"""
)
st.caption(
    """
In this notation:
- $A$ represents the "all_data" dataset.
- $B$ represents the "suitable_baseline_no" dataset.
- $d_{test}$ is the "test_case_date".
- $A'$ and $B'$ are subsets of $A$ and $B$ that only include dates prior to $d_{test}$.
- $C$ is a set of dates in $A'$ not found in $B'$.
- $|C|$ represents the count of elements in set $C$.
- $\max(A')$ is the latest date in $A'$.
"""
)


st.title("calculate_power_averages Function")
st.caption("Calculating average power for each type and time step")
st.latex(
    r"""
1. P_{type,i} = \{p : p \text{ is a power value at time step } t_i\} \quad \text{for each type and } i \in \{1,2,\dots,96\} \\
2. A_{type,i} = \frac{1}{|P_{type,i}|}\sum_{p \in P_{type,i}} p \quad \text{for each type and } i \in \{1,2,\dots,96\}
"""
)
st.caption(
    """
In this notation:
- $P_{type,i}$ represents the set of power values at time step $t_i$ for a specific power type (main, ahu, or solar).
- $p$ represents a power value in the set $P_{type,i}$.
- $|P_{type,i}|$ represents the count of elements in set $P_{type,i}$.
- $A_{type,i}$ represents the average power at time step $t_i$ for a specific power type.
"""
)



"""
open-fdd/                # Repository root
├── open_fdd/            # Python package root
│   ├── __init__.py
│   ├── air_handling_unit/
│   │   ├── __init__.py
│   │   ├── faults/
│   │   │   ├── __init__.py
│   │   │   ├── helper_utils.pya
│   │   │   ├── fault_condition.py
│   │   │   ├── fault_condition_one.py
│   │   │   ├── fault_condition_two.py
│   │   │   └── ... # other fault conditions
│   │   ├── examples/
│   │   ├── reports/
│   │   │   ├── __init__.py
│   │   │   ├── base_report.py
│   │   │   ├── report_fc1.py
│   │   │   ├── report_fc2.py
│   │   │   └── ... # other reports
│   │   └── images/
│   │       ├── fc1_definition.png
│   │       ├── fc2_definition.png
│   │       └── ... # other images
│   ├── black_box/
│   │   └── ... # subject to change being experimental
├── tests/
│   ├── __init__.py
│   ├── air_handling_unit/
│   │   ├── __init__.py
│   │   ├── test_ahu_fc1.py
│   │   ├── test_ahu_fc2.py
│   │   └── ... # other tests
├── setup.py
├── README.md
├── LICENSE
└── requirements.txt

"""



import pandas as pd
import numpy as np
from open_fdd.air_handling_unit.faults.fault_condition import FaultCondition
import sys


class FaultConditionTwo(FaultCondition):
    """Class provides the definitions for Fault Condition 2.
    Mix temperature too low; should be between outside and return air.
    """

    def __init__(self, dict_):
        """
        :param dict_:
        """
        self.mix_degf_err_thres = float
        self.return_degf_err_thres = float
        self.outdoor_degf_err_thres = float
        self.mat_col = str
        self.rat_col = str
        self.oat_col = str
        self.supply_vfd_speed_col = str
        self.troubleshoot_mode = bool  # default to False
        self.rolling_window_size = int

        self.set_attributes(dict_)

    def apply(self, df: pd.DataFrame) -> pd.DataFrame:
        if self.troubleshoot_mode:
            self.troubleshoot_cols(df)

        # Check analog outputs [data with units of %] are floats only
        columns_to_check = [self.supply_vfd_speed_col]
        self.check_analog_pct(df, columns_to_check)

        # Fault condition-specific checks / flags
        df["mat_check"] = df[self.mat_col] + self.mix_degf_err_thres
        df["temp_min_check"] = np.minimum(
            df[self.rat_col] - self.return_degf_err_thres,
            df[self.oat_col] - self.outdoor_degf_err_thres,
        )

        df["combined_check"] = (df["mat_check"] < df["temp_min_check"]) & (
            df[self.supply_vfd_speed_col] > 0.01
        )

        # Rolling sum to count consecutive trues
        rolling_sum = (
            df["combined_check"].rolling(window=self.rolling_window_size).sum()
        )
        # Set flag to 1 if rolling sum equals the window size
        df["fc2_flag"] = (rolling_sum >= self.rolling_window_size).astype(int)

        if self.troubleshoot_mode:
            print("Troubleshoot mode enabled - not removing helper columns")
            sys.stdout.flush()
            del df["mat_check"]
            del df["temp_min_check"]
            del df["combined_check"]

        return df



import pandas.api.types as pdtypes
from open_fdd.air_handling_unit.faults.helper_utils import HelperUtils
import sys


class FaultCondition:
    """Parent class for Fault Conditions. Methods are inherited to all children."""

    def set_attributes(self, dict_):
        """Passes dictionary into initialization of class instance, then uses the attributes called out below in
        attributes_dict to set only the attributes that match from dict_.

        :param dict_: dictionary of all possible class attributes (loaded from config file)
        """
        for attribute in self.__dict__:
            upper = attribute.upper()
            value = dict_[upper]
            self.__setattr__(attribute, value)

    def troubleshoot_cols(self, df):
        """print troubleshoot columns mapping

        :param df:
        :return:
        """
        print("Troubleshoot mode enabled - not removing helper columns")
        for col in df.columns:
            print(
                "df column: ",
                col,
                "- max: ",
                df[col].max(),
                "- col type: ",
                df[col].dtypes,
            )
            sys.stdout.flush()

    def check_analog_pct(self, df, columns):
        """check analog outputs [data with units of %] are floats only

        :param columns:
        :return:
        """
        helper = HelperUtils()
        for col in columns:
            if not pdtypes.is_float_dtype(df[col]):
                df = helper.convert_to_float(df, col)
            if df[col].max() > 1.0:
                raise TypeError(helper.float_max_check_err(col))
